package cn.skcks.docking.gb28181.wvp.service.wvp;

import cn.hutool.crypto.digest.MD5;
import cn.skcks.docking.gb28181.common.json.JsonResponse;
import cn.skcks.docking.gb28181.wvp.config.WvpProxyConfig;
import cn.skcks.docking.gb28181.wvp.dto.login.WvpLoginReq;
import cn.skcks.docking.gb28181.wvp.dto.login.WvpLoginResp;
import cn.skcks.docking.gb28181.wvp.proxy.WvpProxyClient;
import cn.skcks.docking.gb28181.wvp.service.download.DownloadService;
import com.github.rholder.retry.Retryer;
import com.github.rholder.retry.RetryerBuilder;
import com.github.rholder.retry.StopStrategies;
import com.github.rholder.retry.WaitStrategies;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

@Slf4j
@Service
@RequiredArgsConstructor
public class WvpService {
    private final WvpProxyClient wvpProxyClient;
    private final WvpProxyConfig wvpProxyConfig;

    private final DownloadService downloadService;

    @SneakyThrows
    public void video(HttpServletRequest request, HttpServletResponse response) {
        Retryer<JsonResponse<?>> retryer = RetryerBuilder.<JsonResponse<?>>newBuilder()
                // 异常就重试
                .retryIfException()
                // 重试间隔
                .withWaitStrategy(WaitStrategies.fixedWait(5, TimeUnit.SECONDS))
                // 重试次数
                .withStopStrategy(StopStrategies.stopAfterAttempt(5))
                .retryIfResult(result -> result == null || (result.getCode() != 0 && result.getCode() != 200))
                .build();

        String passwdMd5 = MD5.create().digestHex(wvpProxyConfig.getPasswd());
        WvpLoginReq loginReq = WvpLoginReq.builder()
                .username(wvpProxyConfig.getUser())
                .password(passwdMd5)
                .build();

        retryer.call(()->{
            JsonResponse<WvpLoginResp> login = wvpProxyClient.login(loginReq);
            String accessToken = login.getData().getAccessToken();
            log.info("wvp 登录成功 accessToken => {}", accessToken);

            downloadService.download(request,response,"http://192.168.1.241:18979/download/recordTemp/0490d767d94ce20aedce57c862b6bfe9/rtp/59777645.mp4");
            return login;
        });
    }
}
